% code based upon Plett, Gregory L., "Battery
% Management Systems, Volume II, Equivalent-Circuit Methods," Artech House, 
% 2015.

% GPs code was updated for better author understanding


function results = iterSimVehicleBK(deltaT,desSpeedSI,gradeInPercent,k,length,OCVAtEvery0Point1PercentSoC,numLoops,vehicle)

    persistent prevBattSoCNoRegen prevBattSoCWithCurrentSenseError prevBattSoCWithRegenAndBias prevBattSoCWithRegen prevActualSpeed prevDistance prevLimMotorSpeedRPM; % remain in memory between function calls, only visible to this function

    if(isempty(prevActualSpeed)) % need to initialize persistent variables
       disp('Is empty!');
       prevBattSoCNoRegen = vehicle.drivetrain.pack.fullSoC; % [%], start at fully loaded SoC 
       prevBattSoCWithCurrentSenseError = vehicle.drivetrain.pack.fullSoC; % [%], start at fully loaded SoC 
       prevBattSoCWithRegen = vehicle.drivetrain.pack.fullSoC; % [%], start at fully loaded SoC 
       prevBattSoCWithRegenAndBias = vehicle.drivetrain.pack.fullSoC;
       prevActualSpeed = 0; % vehicle speed, [m/s]
       prevDistance = 0;
       prevLimMotorSpeedRPM = 0; % previous limited motor speed [RPM]
    end
    
    %% initialize results struct
    
    results.actualAccel = 0;
    results.actualSpeed = 0;
    results.aeroForce = 0;
    results.battPower = 0;
    results.battSoCWithRegen = 0;
    results.battSoCNoRegen = 0;
    results.battCurrent = 0;
    results.battVCell = 0;
    results.battVoltagePack = 0;
    results.demandMotorTorque = 0;
    results.desAccelForce = 0;
    results.distance = 0;
    results.gradeForce = 0;
    results.limMotorSpeedRPM = 0;
    results.limMotorTorque = 0;
    results.motorPower = 0;
    results.rollingForce = 0;

    %% desired speed to actual speed calculations
    
    desAccel = (desSpeedSI - prevActualSpeed)/(deltaT); % des: desired, curr: current, prev: previous
    results.desAccelForce = vehicle.equivMass*desAccel; % desired acceleration force [m/s]

    rho_air = 1.225; % [kg/m^3], air density at 0^o C and 1 atm, see wikipedia, density of air
    %disp('prevActualSpeed: '); disp(prevActualSpeed);
    results.aeroForce = 0.5*rho_air*vehicle.frontalArea*vehicle.Cd*(prevActualSpeed^2); % rho_air: air density, Cd: drag coefficient
    g = 9.81; % [m/s^2]

    if(abs(prevActualSpeed)>1e-15) % do not use desired speed because it could be the motor cannot achieve that speed, assume very small
        % speeds do not have a rolling force!
       results.rollingForce = vehicle.wheel.Cr*vehicle.maxVehicleMass*g; % [N]
    else
       results.rollingForce = 0; % [N] 
    end

    gradeAngleInRads = atan(gradeInPercent/100);
    results.gradeForce = vehicle.maxVehicleMass*g*sin(gradeAngleInRads); % gradeAngleInRads: slope of road, incline: positive, descent: negative
    
    brakeDrag = 0; % assume no frictional losses from brakes

    results.demandMotorTorque = (vehicle.wheel.radius/vehicle.drivetrain.N)*(results.desAccelForce+results.aeroForce+results.rollingForce+results.gradeForce+brakeDrag);

    if(abs(prevLimMotorSpeedRPM)>=0 && abs(prevLimMotorSpeedRPM)<=vehicle.drivetrain.motor.ratedRPM) % straight line of torque vs speed curve
        maxMotorTorque = vehicle.drivetrain.motor.maxRatedTorque; % max motor torque is equal to straight line value
    else % rated RPM < demanded motor torque <= max motor RPM, actual motor speed limited in a later equation
        maxMotorTorque = vehicle.drivetrain.motor.maxRatedTorque*(vehicle.drivetrain.motor.ratedRPM/prevLimMotorSpeedRPM);
    end
    
    if(results.demandMotorTorque>=0)
        results.limMotorTorque = min(results.demandMotorTorque,maxMotorTorque); % limited motor torque [Nm]
    else % negative demanded motor torque
        maxRegenMotorTorque = -min(maxMotorTorque,vehicle.drivetrain.motor.regenFraction*vehicle.drivetrain.motor.maxRatedTorque);
        results.limMotorTorque = max(results.demandMotorTorque,maxRegenMotorTorque); % limited motor torque [Nm]
    end
    
    actualAccelForce = results.limMotorTorque*(vehicle.drivetrain.N/vehicle.wheel.radius)-results.aeroForce-results.rollingForce-results.gradeForce; % actual acceleration force [N]
    
    results.actualAccel = actualAccelForce/vehicle.equivMass; % actual acceleration [m/s^2]
    
    testSpeed = prevActualSpeed+results.actualAccel*deltaT;
    
    motorTestSpeed = testSpeed*vehicle.drivetrain.N*60/(2*pi*vehicle.wheel.radius);
    
    if(motorTestSpeed>=0)
       results.limMotorSpeedRPM = min(motorTestSpeed,vehicle.drivetrain.motor.maxRatedRPM);
    else % motorTestSpeed < 0
       results.limMotorSpeedRPM = max(motorTestSpeed,-vehicle.drivetrain.motor.maxRatedRPM);
    end
    
    results.actualSpeed = results.limMotorSpeedRPM*2*pi*vehicle.wheel.radius/(60*vehicle.drivetrain.N); % [m/s]
    
    
    DeltaX = (results.actualSpeed+prevActualSpeed)/2*(deltaT)/1000; % [km]
%     disp('DeltaX: '); disp(DeltaX);
    results.distance = prevDistance + DeltaX; % [km]
    
    %% motor power to SoC calculations
    
    results.motorPower = (2*pi*(results.limMotorSpeedRPM+prevLimMotorSpeedRPM)*results.limMotorTorque)/(2*60*1000); % [kW]
    
    if(results.motorPower>=0) % motor is standing still or accelerating
        results.battPower = vehicle.overheadPower + results.motorPower/vehicle.drivetrain.efficiency; % battery power, battery is discharging
    else % motor power is negative, regen
        results.battPower = vehicle.overheadPower + results.motorPower*vehicle.drivetrain.efficiency; % battery power, power is charging
    end
    
    vehicle.drivetrain.pack.module.cell.vcurrent = findOCVAtZk(OCVAtEvery0Point1PercentSoC,prevBattSoCWithRegen/100); % assume battery voltage = OCV, finds this using previous estimated SoC
    results.battVCell = vehicle.drivetrain.pack.module.cell.vcurrent;
    results.battVoltagePack = vehicle.drivetrain.pack.numSeries*vehicle.drivetrain.pack.module.numSeries*results.battVCell;
    results.battVoltagePackWithNoise = vehicle.drivetrain.pack.numSeries*vehicle.drivetrain.pack.module.numSeries*results.battVCell*(1+wgn(1,1,-60,'real')); % (-40 -> 1% std dev, -46 -> 0.5% std dev, -60 -> 0.1% std dev)

    %results.battVoltagePack = vehicle.drivetrain.pack.numSeries*vehicle.drivetrain.pack.module.numSeries*results.battVCell*(1+wgn(1,1,-100,'real')); % adds about a 1% voltage innacuracy, white noise
    % results.battCurrent = results.battPower*1000/results.battVoltagePack; % battery current > 0 -> current discharge
    results.battCurrent = (results.battPower*1000/results.battVoltagePack); % add white noise to battery current 
    
    results.battCurrentWithBias = results.battCurrent*(1.01); % add constant current bias of 1%
    results.battCurrentWithNoise = ((results.battPower*1000/results.battVoltagePack))*(1+0.01); % add white noise to battery current
    
    % new code
    if(k>length/(2*numLoops))
        results.battCurrentSensorError = results.battCurrent*(1+wgn(1,1,-6,'real')); %results.battCurrent*(1+0.5);%results.battCurrent*(1+wgn(1,1,-6,'real')); % battery current sensor malfunction, -20: 10% std dev.,-10: 30% std dev., -6: 50% std. dev.
    else
        results.battCurrentSensorError = results.battCurrent;
    end
    
    % new code

    results.battSoCWithRegen = prevBattSoCWithRegen - 100*(results.battCurrent*deltaT)/(3600*vehicle.drivetrain.pack.capacity); % coulomb counting, [%]
    results.battSoCWithRegenBias = prevBattSoCWithRegenAndBias - 100*(results.battCurrentWithBias*deltaT)/(3600*vehicle.drivetrain.pack.capacity); % coulomb counting, [%]
    results.battSoCWithCurrentSenseError = prevBattSoCWithCurrentSenseError - 100*(results.battCurrentSensorError*deltaT)/(3600*vehicle.drivetrain.pack.capacity); % coulomb counting, [%]

    
    if(results.battCurrent < 0) % regenerating the battery
        battCurrentNoRegen = 0;
    else % battCurrent >= 0
        battCurrentNoRegen = results.battCurrent;
    end
    
    results.battSoCNoRegen = prevBattSoCNoRegen - 100*(battCurrentNoRegen*deltaT)/(3600*vehicle.drivetrain.pack.capacity); % coulomb counting, [%]
    
    % for next iteration
    prevActualSpeed = results.actualSpeed;
    prevBattSoCWithCurrentSenseError = results.battSoCWithCurrentSenseError;
    prevBattSoCWithRegen = results.battSoCWithRegen;
    prevBattSoCWithRegenAndBias = results.battSoCWithRegenBias;
    prevBattSoCNoRegen = results.battSoCNoRegen;
    prevDistance = results.distance;
    prevLimMotorSpeedRPM = results.limMotorSpeedRPM;
    
    if(k==length) % last iteration
       clear prevBattSoCNoRegen prevBattSoCWithRegen prevActualSpeed prevDistance prevLimMotorSpeedRPM; % persistent variables should be initialized every time setupAndRunSimVehicleBK.m is run 
    end
    
end