%% import desired speed
clear; clc; close all;

testToRun = 'hwfet'; % choose hwfet, nycc or udds

if(strcmp(testToRun,'hwfet'))
    filename = 'hwfet.txt';
elseif(strcmp(testToRun,'nycc'))
    filename = 'nycc.txt'; 
elseif(strcmp(testToRun,'udds'))
    filename = 'udds.txt';
else
    error('Error. Must choose either hwfet,nycc or udds in testToRun');
end

R1 = 2; % row offset, start reading at test time 0, skips headers
C1 = 0; % column offset
cycle = dlmread(filename,'\t',R1,C1); % format [test time in seconds, test speed in mph]
testTime = cycle(:,1);
desSpeedMPH = cycle(:,2); % desired speed in miles per hour
desSpeedKPH = 1.60934*desSpeedMPH; % desired speed in km per hour
desSpeedSI = desSpeedKPH/3.6; % desired speed in SI units [m/s]

numLoops = 1;
[desSpeedSILoop,timeLooped] = loopSpeedProfile(desSpeedSI,numLoops); % returns timeLooped in seconds

%% load OCV vs SoC relationship

load('paramEstimationOutput.mat'); % includes 'timeEKF','iEKF','vEKF','trueSoC','OCVAtEvery0Point1PercentSoC','thetaHatPlus','eta_25','Q_est'
clear timeEKF iEKF vEKF trueSoC; % not needed

%% initialize drivetrain, motor, vehicle & wheel values

% setup cell
capacity = 15; % [Ah], change capacity
%capacity = Q_est/3600; % [Ah]
vcurrent = 4.2; % [V], start at max voltage
vnom = 3.8; % [V]
weight = 0.45; % [kg]
cell = setupCell(capacity,vcurrent,vnom,weight);
    
% setup module
numParallel = 3; % number of cells in parallel in one module
numSeries = 8; % number of cells in series in one module
overhead = 0.08; % overhead by weight ???
module = setupModule(cell,numParallel,numSeries,overhead);

% setup pack
efficiency = 0.96; % [unitless]
emptySoC = 25; % [%]
fullSoC = 75; % [%]
numSeries = 12; % number of modules in series in one battery pack
overhead = 0.1; % overhead by weight ???
pack = setupPack(efficiency,emptySoC,fullSoC,module,numSeries,overhead);

% setup motor
inertia = 0.2; % kg/m^2
maxRatedRPM = 12000; % [RPM]
maxRatedTorque = 275; % [Nm]
ratedRPM = 4000; % [RPM]
regenFraction = 0.9; % drivetrain.regenTorque in GP setupSimVehicle.m
efficiency = 0.95; % [unitless]
motor = setupMotor(efficiency,inertia,maxRatedRPM,maxRatedTorque,ratedRPM,regenFraction);

% setup drivetrain
gearboxInertia = 0.05; % [kg/m^2]
gearEfficiency = 0.97; % [unitless]
gearRatio = 12;
inverterEfficiency = 0.94;
drivetrain = setupDrivetrain(gearboxInertia,gearEfficiency,gearRatio,inverterEfficiency,motor,pack);

% setup wheel
Cr = 0.0111; % rolling coefficient
radius = 0.35; % [m]
inertia = 8; % [kg/m^2]
wheel = setupWheel(Cr,inertia,radius);

% setup vehicle
Cd = 0.22; % drag coefficient, probably more like 0.285
frontalArea = 1.84; % [m^2], 2.14 m^2 for 2014 chevy volt
overheadPower = 0.2; % [kW]
numWheels = 4;
payload = 75; % [kg], could increase this to 390 kg
weight = 1425; % [kg], 1721 kg
vehicle = setupVehicle(Cd,drivetrain,frontalArea,motor,numWheels,overheadPower,payload,weight,wheel);


length = length(timeLooped);
% initialize results structure
results.actualSpeed = zeros(length,1);
results.aeroForce = zeros(length,1);
results.battSoCNoRegen = zeros(length,1);
results.battSoCWithRegen = zeros(length,1);
results.battCurrent = zeros(length,1);
results.battVCell = zeros(length,1);
results.battVoltagePack = zeros(length,1);
results.demandMotorTorque = zeros(length,1);
results.desAccelForce = zeros(length,1);
results.distance = zeros(length,1);
results.gradeForce = zeros(length,1);
results.limMotorSpeedRPM = zeros(length,1);
results.limMotorTorque = zeros(length,1);
results.motorPower = zeros(length,1);
results.range = 0; % only one scalar is estimated
results.rollingForce = zeros(length,1);
results.timeLooped = zeros(length,1);
results.zEKF = zeros(length,1);

gradeInPercent = 0.3; % positive is incline, negative is descent, gives rise/run in [%]
deltaT = timeLooped(2)-timeLooped(1); % assumes each time sample has same Delta T

SigmaX0 = diag([1e-6 1e-8 1e-6]); % uncertainty of initial state
SigmaW = 2e-1; % Current sensor uncertainty, state equation
SigmaV = 2e-1; % Voltage sensor uncertainty, output equation

packCapacity = vehicle.drivetrain.pack.module.numParallel*Q_est;
ekfDataStruct = initializeEKF(SigmaX0,SigmaV,SigmaW,thetaHatPlus,eta_25,packCapacity);

for k=1:length % start at time=1 sec, prevTime = 0
    resultsIter = iterSimVehicleBK(deltaT,desSpeedSILoop(k),gradeInPercent,k,length,OCVAtEvery0Point1PercentSoC,vehicle);
    results.actualSpeed(k) = resultsIter.actualSpeed;
    results.aeroForce(k) = resultsIter.aeroForce;
    results.battSoCNoRegen(k) = resultsIter.battSoCNoRegen;
    results.battSoCWithRegen(k) = resultsIter.battSoCWithRegen;
    results.battCurrent(k) = resultsIter.battCurrent;
    results.battVCell(k) = resultsIter.battVCell;
    results.battVoltagePack(k) = resultsIter.battVoltagePack;
    results.demandMotorTorque(k) = resultsIter.demandMotorTorque;
    results.desAccelForce(k) = resultsIter.desAccelForce;
    results.distance(k) = resultsIter.distance;
    results.gradeForce(k) = resultsIter.gradeForce;
    results.limMotorSpeedRPM(k) = resultsIter.limMotorSpeedRPM;
    results.limMotorTorque(k) = resultsIter.limMotorTorque;
    results.motorPower(k) = resultsIter.motorPower;
    results.rollingForce(k) = resultsIter.rollingForce;
    
    vk = results.battVCell(k);
    %ik = results.battCurrent(k)/vehicle.drivetrain.pack.module.numParallel;
    ik = results.battCurrent(k);
    [zkEKF,zkBound,vEst,ekfDataStruct] = iterativeEKF(vk,ik,deltaT,ekfDataStruct,OCVAtEvery0Point1PercentSoC);
    results.zEKF(k) = zkEKF;
end

results.timeLooped = timeLooped;
results.desSpeedKPH = desSpeedKPH;

results.range = results.distance*(vehicle.drivetrain.pack.fullSoC-vehicle.drivetrain.pack.emptySoC)/(results.battSoCWithRegen(1)-results.battSoCWithRegen(end));

%disp('Q EKF: '); disp(ekfDataStruct.Q);

%% save different speed profile data

if(strcmp(testToRun,'hwfet'))
    resultsHWFET = results;
    filename = 'HWFETResults.mat';
    save(filename,'resultsHWFET'); 
elseif(strcmp(testToRun,'nycc'))
    resultsNYCC = results;
    filename = 'NYCCResults.mat';
    save(filename,'resultsNYCC'); 
else % udds
    resultsUDDS = results;
    filename = 'UDDSResults.mat';
    save(filename,'resultsUDDS'); 
end

%% import it to plot everything together

%clear; close all;

load HWFETResults;
load NYCCResults;
load UDDSResults;

%% plotting
actSpeedKPH_HWFET = 3.6*resultsHWFET.actualSpeed; % convert from m/s to km/hour
actSpeedKPH_NYCC = 3.6*resultsNYCC.actualSpeed; % convert from m/s to km/hour
actSpeedKPH_UDDS = 3.6*resultsUDDS.actualSpeed; % convert from m/s to km/hour

timeLoopedMin_HWFET = resultsHWFET.timeLooped/60; % convert from s to min
timeLoopedMin_NYCC = resultsNYCC.timeLooped/60; % convert from s to min
timeLoopedMin_UDDS = resultsUDDS.timeLooped/60; % convert from s to min

SoCGainedWithRegen_HWFET = resultsHWFET.battSoCWithRegen - resultsHWFET.battSoCNoRegen;
SoCGainedWithRegen_NYCC = resultsNYCC.battSoCWithRegen - resultsNYCC.battSoCNoRegen;
SoCGainedWithRegen_UDDS = resultsUDDS.battSoCWithRegen - resultsUDDS.battSoCNoRegen;

% plot desired and actual speeds

% figure(1); plot(timeLoopedMin_HWFET,resultsHWFET.desSpeedKPH,'k','LineWidth',7); 
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_HWFET,actSpeedKPH_HWFET,'y','LineWidth',1);
% legend('Desired Speed','Actual Speed'); xlabel('Time [min]'); ylabel('Speed [km/h]');
% title('Highway Fuel Efficiency Test (HW-FET) Speed vs Time Profile');
% hold off; % next plot will clear the previous plots if figure(1) is still used
% 
% figure(2); plot(timeLoopedMin_NYCC,resultsNYCC.desSpeedKPH,'k','LineWidth',7); 
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_NYCC,actSpeedKPH_NYCC,'y','LineWidth',1);
% legend('Desired Speed','Actual Speed'); xlabel('Time [min]'); ylabel('Speed [km/h]');
% title('New York City Cycle (NYCC) Speed vs Time Profile');
% hold off; % next plot will clear the previous plots if figure(1) is still used
% 
% figure(3); plot(timeLoopedMin_UDDS,resultsUDDS.desSpeedKPH,'k','LineWidth',7); 
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_UDDS,actSpeedKPH_UDDS,'y','LineWidth',1);
% legend('Desired Speed','Actual Speed'); xlabel('Time [min]'); ylabel('Speed [km/h]');
% title('Urban Dynanometer Driving Schedule (UDDS) Speed vs Time Profile');
% hold off; % next plot will clear the previous plots if figure(1) is still used


% plot SoC with the desired speed profiles, with and without regen, plot
% "error" between the two also

% figure(4); plot(timeLoopedMin_HWFET,resultsHWFET.battSoCWithRegen); 
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_NYCC,resultsNYCC.battSoCWithRegen);
% plot(timeLoopedMin_UDDS,resultsUDDS.battSoCWithRegen);
% legend('HW-FET Regen','NYCC Regen','UDDS Regen'); xlabel('Time [min]'); ylabel('SoC [%]');
% title('SoC of Desired Speed Profiles With Regen Braking');
% hold off; % next plot will clear the previous plots if figure(1) is still used
% 
% figure (5); plot(timeLoopedMin_HWFET,resultsHWFET.battSoCNoRegen);
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_NYCC,resultsNYCC.battSoCNoRegen);
% plot(timeLoopedMin_UDDS,resultsUDDS.battSoCNoRegen);
% legend('HW-FET No Regen','NYCC No Regen','UDDS No Regen'); xlabel('Time [min]'); ylabel('SoC [%]');
% title('SoC of Desired Speed Profiles No Regen Braking');
% hold off; % next plot will clear the previous plots if figure(1) is still used



% figure (6); plot(timeLoopedMin_HWFET,SoCGainedWithRegen_HWFET);
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_NYCC,SoCGainedWithRegen_NYCC);
% plot(timeLoopedMin_UDDS,SoCGainedWithRegen_UDDS);
% legend('HW-FET','NYCC','UDDS'); xlabel('Time [min]'); ylabel('SoC Gained [%]');
% title('SoC Gained With Regen of Desired Speed Profiles');
% hold off; % next plot will clear the previous plots if figure(1) is still used

% check own results with Plett's results

% load resultsTest
% figure(7); plot(timeLooped,results.battSoCWithRegen,timeLooped,results.battSoCNoRegen);
% title('State of Charge for the NYCC Speed Profile'); legend('Regen','No Regen');
% xlabel('Time [s]'); ylabel('SoC [%]');

% figure(8); plot(timeLooped,results.battCurrent,timeLooped,resultsGP.current); legend('BK','GP');
% title('Battery Current');
%figure(9); plot(timeLooped,results.actualSpeed, timeLooped,resultsGP.actualSpeed); legend('BK','GP');

% figure(10); plot(timeLooped,results.battVoltagePack); title('Battery Pack Voltage');
% 
% figure(11); plot(timeLooped,results.battVCell); title('Battery Cell Voltage');

% figure(12); plot(resultsHWFET.timeLooped,resultsHWFET.battSoCWithRegen,'r','LineWidth',5); hold on;
% plot(resultsHWFET.timeLooped,100*resultsHWFET.zEKF,'g','LineWidth',2);
% purple = [0.55 0.26 0.51]; % find color using gimp, find RGB values -> divide by 255 to get value between 0 and 1
% % see http://www.tigercolor.com/color-lab/color-theory/color-harmonies.htm
% plot(resultsNYCC.timeLooped,resultsNYCC.battSoCWithRegen,'Color',purple,'LineWidth',5);
% plot(resultsNYCC.timeLooped,100*resultsNYCC.zEKF,'y','LineWidth',2);
% plot(resultsUDDS.timeLooped,resultsUDDS.battSoCWithRegen,'b','LineWidth',5);
% orange = [.95 .76 .35];
% plot(resultsUDDS.timeLooped,100*resultsUDDS.zEKF,'Color',orange,'LineWidth',2);
% title('SoC using Coulomb Counting and EKF for three different speed profiles');
% legend('HW-FET Coulomb Counting','HW-FET EKF','NYCC Coulomb Counting','NYCC EKF','UDDS Coulomb Counting','UDDS EKF');
% xlabel('Time [s]'); ylabel('SoC [%]');
% hold off;

figure(13); plot(timeLoopedMin_HWFET,resultsHWFET.distance);
hold on; % next plot will not overwrite last plot
plot(timeLoopedMin_NYCC,resultsNYCC.distance);
plot(timeLoopedMin_UDDS,resultsUDDS.distance);
title('Distance Calculations for Desired Speed Profiles');
legend('HW-FET','NYCC','UDDS');
xlabel('Time [min]'); ylabel('Distance [km]');
