% code based upon Plett, Gregory L., "Battery
% Management Systems, Volume II, Equivalent-Circuit Methods," Artech House, 
% 2015.

% GPs code was updated for better author understanding

clear; clc; close all;

%% load OCV vs SoC relationship

load('paramEstimationOutput.mat'); % includes 'timeEKF','iEKF','vEKF','trueSoC','OCVAtEvery0Point1PercentSoC','thetaHatPlus','eta_25','Q_est'
load('speedProfileStructsGradeAngle'); % loads different grade angles with a length = different speed profiles (NYCC,UDDS,HWFET)
%clear timeEKF iEKF vEKF trueSoC; % not needed

%% initialize drivetrain, motor, vehicle & wheel values

% setup cell
%capacity = 15; % [Ah], 2013 Chevy Volt, pack capacity of 45 Ah, see (https://avt.inl.gov/sites/default/files/pdf/phev/fact2013chevroletvolt1.pdf)
capacity = Q_est/3600; % [Ah]
vcurrent = 4.2; % [V], start at max voltage
vnom = 3.7; % [V]
weight = 0.45; % [kg]
battCell = setupCell(capacity,vcurrent,vnom,weight);
    
% setup module
numParallel = 3; % number of cells in parallel in one module
numSeries = 8; % number of cells in series in one module
overhead = 0.08; % overhead by weight ???
module = setupModule(battCell,numParallel,numSeries,overhead);

% setup pack
efficiency = 0.96; % [unitless]
emptySoC = 25; % [%]
fullSoC = 75; % [%]
numSeries = 12; % number of modules in series in one battery pack
overhead = 0.1; % overhead by weight ???
pack = setupPack(efficiency,emptySoC,fullSoC,module,numSeries,overhead);

% setup motor
inertia = 0.2; % kg/m^2
maxRatedRPM = 12000; % [RPM]
maxRatedTorque = 275; % [Nm]
ratedRPM = 4000; % [RPM]
regenFraction = 0.9; % drivetrain.regenTorque in GP setupSimVehicle.m
efficiency = 0.95; % [unitless]
motor = setupMotor(efficiency,inertia,maxRatedRPM,maxRatedTorque,ratedRPM,regenFraction);

% setup drivetrain
gearboxInertia = 0.05; % [kg/m^2]
gearEfficiency = 0.97; % [unitless]
gearRatio = 12;
inverterEfficiency = 0.94;
drivetrain = setupDrivetrain(gearboxInertia,gearEfficiency,gearRatio,inverterEfficiency,motor,pack);

% setup wheel
Cr = 0.0111; % rolling coefficient
radius = 0.35; % [m]
inertia = 8; % [kg/m^2]
wheel = setupWheel(Cr,inertia,radius);

% setup vehicle
Cd = 0.22; % drag coefficient, probably more like 0.285
frontalArea = 1.84; % [m^2], 2.14 m^2 for 2014 chevy volt
overheadPower = 0.2; % [kW]
numWheels = 4;
payload = 75; % [kg], could increase this to 390 kg
weight = 1425; % [kg], 1721 kg
vehicle = setupVehicle(Cd,drivetrain,frontalArea,motor,numWheels,overheadPower,payload,weight,wheel);

%% initialize the rest

%gradeInPercent = 0.3; % positive is incline, negative is descent, gives rise/run in [%]
% SigmaX0 = diag([1e-6 1e-8 1e-6]); % uncertainty of initial state, [ir,hk,zk]
% SigmaX0 = diag([1e-6 1e-8 1e-4]); % uncertainty of initial state, [ir,hk,zk]
SigmaX0 = diag([1e-6 1e-8 1e-4;]); % uncertainty of initial state, [ir,hk,zk]
SigmaW = 2; % Current sensor uncertainty, state equation, 2e-1
SigmaV = 1; % Voltage sensor uncertainty, output equation

%packCapacity = vehicle.drivetrain.pack.module.numParallel*Q_est/3600; % Ah
%packCapacity = vehicle.drivetrain.pack.module.numParallel*15;

% ekfDataStruct = initializeEKF(SigmaX0,SigmaV,SigmaW,thetaHatPlus,eta_25,packCapacity);


%% import the drive profiles and run the simulations

driveProfiles = {'hwfet.txt','nycc.txt','udds.txt'}; % drive profiles text cell array
%driveProfiles = {'hwfet.txt','udds.txt','nycc.txt'}; % drive profiles text cell array
%driveProfiles = {'googleMapsTest.txt'}; % drive profiles text cell array
driveProfilesResults = cell(1,length(driveProfiles)); % empty cell array that will store the results of the driveProfiles

R1 = 2; % row offset, start reading at test time 0, skips headers
C1 = 0; % column offset


for i=1:length(driveProfiles)
    %disp('i: '); disp(i);
    cycle = dlmread(driveProfiles{i},'\t',R1,C1); % format [test time in seconds, test speed in mph], tab deliminator
    testTime = cycle(:,1);
    desSpeedMPH = cycle(:,2); % desired speed in miles per hour
    desSpeedKPH = 1.60934*desSpeedMPH; % desired speed in km per hour
    desSpeedSI = desSpeedKPH/3.6; % desired speed in SI units [m/s]
    
    numLoops = [1 1 1]; %[3 12 3];
    [desSpeedSILoop,timeLooped] = loopSpeedProfile(desSpeedSI,numLoops(i)); % returns timeLooped in seconds
    
    deltaT = timeLooped(2)-timeLooped(1); % assumes each time sample has same Delta T
    
%     if(i==2)
%         SigmaX0 = 6.8e-4;
%     elseif(i==3)
%         SigmaX0 = 2.7e-4;
%     end
    

    
    len = length(timeLooped);
    fprintf('length time: %i \n',len);
    % initialize results structure
    results.actualAccel = zeros(len,1);
    results.actualSpeed = zeros(len,1);
    results.aeroForce = zeros(len,1);
    results.battCurrent = zeros(len,1);
    results.battCurrentSensorError = zeros(len,1);
    results.battCurrentWithBias = zeros(len,1);
    results.battCurrentWithNoise = zeros(len,1);
    results.battPower = zeros(len,1);
    results.battSoCNoRegen = zeros(len,1);
    results.battSoCWithCurrentSenseError = zeros(len,1);
    results.battSoCWithRegen = zeros(len,1);
    results.battSoCWithRegenBias = zeros(len,1);
    results.battVCell = zeros(len,1);
    results.battVoltagePack = zeros(len,1);
    results.battVoltagePackWithNoise = zeros(len,1);
    results.demandMotorTorque = zeros(len,1);
    results.desAccelForce = zeros(len,1);
    results.distance = zeros(len,1);
    results.eigValueZk = zeros(len,1);
    results.gradeForce = zeros(len,1);
    results.limMotorSpeedRPM = zeros(len,1);
    results.limMotorTorque = zeros(len,1);
    results.motorPower = zeros(len,1);
    results.range = 0; % only one scalar is estimated
    results.rollingForce = zeros(len,1);
    results.sigmaY = zeros(len,1); % voltage error covariance, uncertainty of voltage
    results.timeLooped = zeros(len,1);
    results.timeToGo = 0;
    results.vEst = zeros(len,1);
    results.zkBound = zeros(len,1);
    results.zEKF = zeros(len,1);
    
%     gradeInPercent = ones(length(desSpeedSILoop));
%     
%     for j=1:length(gradeInPercent)
%        gradeInPercent(i) = gradeInPercent(i)*rand(); 
%     end

    if(i==1)
        gradeInPercent = downhillStruct.HWFET;
    elseif(i==2)
        gradeInPercent = downhillStruct.NYCC;
    else % i==3
        gradeInPercent = downhillStruct.UDDS;
    end

    %gradeInPercent = zeros(length(desSpeedSILoop));
 
    %SOC0=1;
    ekfDataStruct = initializeEKF_BK(SigmaX0,SigmaV,SigmaW,fullSoC/100,thetaHatPlus,eta_25,vehicle);

    
    for k=1:len % start at time=1 sec, prevTime = 0
        % added white noise to iterSimVehicleBK Voltage
        resultsIter = iterSimVehicleBK(deltaT,desSpeedSILoop(k),gradeInPercent(k),k,len,OCVAtEvery0Point1PercentSoC,numLoops(i),vehicle);
        results.actualAccel(k) = resultsIter.actualAccel;
        results.actualSpeed(k) = resultsIter.actualSpeed;
        results.aeroForce(k) = resultsIter.aeroForce;
        results.battPower(k) = resultsIter.battPower;
        results.battSoCNoRegen(k) = resultsIter.battSoCNoRegen;
        results.battSoCWithCurrentSenseError(k) = resultsIter.battSoCWithCurrentSenseError;
        results.battSoCWithRegen(k) = resultsIter.battSoCWithRegen;
        results.battSoCWithRegenBias(k) = resultsIter.battSoCWithRegenBias;
        results.battCurrent(k) = resultsIter.battCurrent;
        results.battCurrentWithBias(k) = resultsIter.battCurrentWithBias;
        results.battCurrentWithNoise(k) = resultsIter.battCurrentWithNoise;
        results.battCurrentSensorError(k) = resultsIter.battCurrentSensorError;
        results.battVCell(k) = resultsIter.battVCell;
        results.battVoltagePack(k) = resultsIter.battVoltagePack;
        results.battVoltagePackWithNoise(k) = resultsIter.battVoltagePackWithNoise;
        results.demandMotorTorque(k) = resultsIter.demandMotorTorque;
        results.desAccelForce(k) = resultsIter.desAccelForce;
        results.distance(k) = resultsIter.distance;
        results.gradeForce(k) = resultsIter.gradeForce;
        results.limMotorSpeedRPM(k) = resultsIter.limMotorSpeedRPM;
        results.limMotorTorque(k) = resultsIter.limMotorTorque;
        results.motorPower(k) = resultsIter.motorPower;
        results.rollingForce(k) = resultsIter.rollingForce;
        


        %vk = results.battVoltagePackWithNoise(k);
        vk = results.battVoltagePack(k);
        ik = results.battCurrentSensorError(k); % assume the whole battery pack is one battery to be able to use EKF
        %ik = results.battCurrentWithNoise(k); % battery current with white noise
        %ik = results.battCurrent(k); % battery current with white noise
        
        %fprintf('battVoltPack %3.6f ,battCurrent %3.6 \n',vk,ik);
                
%         fprintf('k: %i s \n',k);
        [zkEKF,zkBound,ekfDataStruct,vEst,SigmaY,eigValueZk] = iterativeEKF_BK(vk,i,ik,k,deltaT,ekfDataStruct,OCVAtEvery0Point1PercentSoC); % i is the speed profile number
        %disp('SigmaX: '); disp(ekfDataStruct.SigmaX);
        results.sigmaY(k) = SigmaY;
        results.vEst(k) = vEst;
        results.zEKF(k) = zkEKF;
        results.zkBound(k) = zkBound;
        results.eigValueZk(k) = eigValueZk;
    end

    results.timeLooped = timeLooped;
    results.desSpeedKPH = desSpeedKPH;

    results.range = results.distance(end)*(vehicle.drivetrain.pack.fullSoC-vehicle.drivetrain.pack.emptySoC)/(vehicle.drivetrain.pack.fullSoC-results.battSoCWithRegen(end));
    results.timeToGo = results.timeLooped(end)*(vehicle.drivetrain.pack.fullSoC-vehicle.drivetrain.pack.emptySoC)/(vehicle.drivetrain.pack.fullSoC-results.battSoCWithRegen(end));
    
    %disp(results);
    
    driveProfilesResults{i} = results; % store all the drive profile results in a cell array  
end

ekfTrueSoC_HWFET = driveProfilesResults{1}.zEKF;
ekfTrueSoC_NYCC = driveProfilesResults{2}.zEKF;
ekfTrueSoC_UDDS = driveProfilesResults{3}.zEKF;


ekfTrueSoCErrorBounds_HWFET = driveProfilesResults{1}.zkBound;
ekfTrueSoCErrorBounds_NYCC = driveProfilesResults{2}.zkBound;
ekfTrueSoCErrorBounds_UDDS = driveProfilesResults{3}.zkBound;

% 
% filename = 'resultsEKFSoCTrue';
% save(filename,'ekfTrueSoC_HWFET','ekfTrueSoC_NYCC','ekfTrueSoC_UDDS');

%% plotting
actSpeedKPH_HWFET = 3.6*driveProfilesResults{1}.actualSpeed; % convert from m/s to km/hour
actSpeedKPH_NYCC = 3.6*driveProfilesResults{2}.actualSpeed; % convert from m/s to km/hour
actSpeedKPH_UDDS = 3.6*driveProfilesResults{3}.actualSpeed; % convert from m/s to km/hour
% % 
timeLoopedMin_HWFET = driveProfilesResults{1}.timeLooped/60; % convert from s to min
timeLoopedMin_NYCC = driveProfilesResults{2}.timeLooped/60; % convert from s to min
timeLoopedMin_UDDS = driveProfilesResults{3}.timeLooped/60; % convert from s to min

SoCGainedWithRegen_HWFET = driveProfilesResults{1}.battSoCWithRegen - driveProfilesResults{1}.battSoCNoRegen;
SoCGainedWithRegen_NYCC = driveProfilesResults{2}.battSoCWithRegen - driveProfilesResults{2}.battSoCNoRegen;
SoCGainedWithRegen_UDDS = driveProfilesResults{3}.battSoCWithRegen - driveProfilesResults{3}.battSoCNoRegen;

%% plot current

% figure(22); plot(driveProfilesResults{1}.battCurrent); title('HWFET');
% figure(23); plot(driveProfilesResults{2}.battCurrent); title('NYCC');
% figure(24); plot(driveProfilesResults{3}.timeLooped/60,driveProfilesResults{3}.battCurrent); title('UDDS Current Profile','FontSize',18);
% xlabel('Time [min]','FontSize',18); ylabel('Current [A]','FontSize',18); set(gca,'FontSize',18);
% 
% fprintf('battcurrent mean HWFET: %3.3f \n',mean(driveProfilesResults{1}.battCurrent));
% fprintf('battcurrent mean NYCC: %3.3f \n',mean(driveProfilesResults{2}.battCurrent));
% fprintf('battcurrent mean UDDS: %3.3f \n',mean(driveProfilesResults{3}.battCurrent));
% 
% fprintf('battcurrent sum HWFET: %3.3f \n',sum(driveProfilesResults{1}.battCurrent));
% fprintf('battcurrent sum NYCC: %3.3f \n',sum(driveProfilesResults{2}.battCurrent));
% fprintf('battcurrent sum UDDS: %3.3f \n',sum(driveProfilesResults{3}.battCurrent));

%% plot desired and actual speeds

% fig1 = figure(1); plot(timeLoopedMin_HWFET,driveProfilesResults{1}.desSpeedKPH,'r','LineWidth',2); 
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_HWFET,actSpeedKPH_HWFET,'b','LineStyle','--','LineWidth',2);
% h_legend = legend('Desired Speed','Actual Speed'); set(h_legend,'FontSize',14);
% xlabel('Time [min]','FontSize',16); ylabel('Speed [km/h]','FontSize',16);
% title('Highway Fuel Efficiency Test (HW-FET) Speed vs Time Profile','FontSize',16);
% %magnifyOnFigure(fig1,'mode','interactive');
% hold off; % next plot will clear the previous plots if figure(1) is still used
% 
% figure(2); plot(timeLoopedMin_NYCC,driveProfilesResults{2}.desSpeedKPH,'r','LineWidth',2); 
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_NYCC,actSpeedKPH_NYCC,'b','LineStyle','--','LineWidth',2);
% h_legend1 = legend('Desired Speed','Actual Speed'); set(h_legend1,'FontSize',14);
% xlabel('Time [min]','FontSize',16); ylabel('Speed [km/h]','FontSize',16);
% title('New York City Cycle (NYCC) Speed vs Time Profile','FontSize',16);
% hold off; % next plot will clear the previous plots if figure(1) is still used
% 
% figure(3); plot(timeLoopedMin_UDDS,driveProfilesResults{3}.desSpeedKPH,'r','LineWidth',2); 
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_UDDS,actSpeedKPH_UDDS,'b','LineStyle','--','LineWidth',2);
% h_legend2 = legend('Desired Speed','Actual Speed'); set(h_legend2,'FontSize',14);
% xlabel('Time [min]','FontSize',16); ylabel('Speed [km/h]','FontSize',16);
% title('Urban Dynanometer Driving Schedule (UDDS) Speed vs Time Profile','FontSize',16);
% hold off; % next plot will clear the previous plots if figure(1) is still used


%% plot SoC with the desired speed profiles, with and without regen, plot "error" between the two also

% figure(4); plot(timeLoopedMin_HWFET,driveProfilesResults{1}.battSoCWithRegen); 
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_NYCC,driveProfilesResults{2}.battSoCWithRegen);
% plot(timeLoopedMin_UDDS,driveProfilesResults{3}.battSoCWithRegen);
% legend('HW-FET Regen','NYCC Regen','UDDS Regen'); xlabel('Time [min]','FontSize',16); ylabel('SoC [%]','FontSize',16);
% title('SoC With Regen Braking','FontSize',18);
% hold off; % next plot will clear the previous plots if figure(1) is still used
% 
% figure (5); plot(timeLoopedMin_HWFET,driveProfilesResults{1}.battSoCNoRegen);
% hold on; % ensures that the next plot does not delete the first plot
% plot(timeLoopedMin_NYCC,driveProfilesResults{2}.battSoCNoRegen);
% plot(timeLoopedMin_UDDS,driveProfilesResults{3}.battSoCNoRegen);
% legend('HW-FET No Regen','NYCC No Regen','UDDS No Regen'); xlabel('Time [min]','FontSize',16); ylabel('SoC [%]','FontSize',16);
% title('SoC No Regen Braking','FontSize',18);
% hold off; % next plot will clear the previous plots if figure(1) is still used
% 
% 
% 
figure (100); plot(timeLoopedMin_HWFET,SoCGainedWithRegen_HWFET);
hold on; % ensures that the next plot does not delete the first plot
plot(timeLoopedMin_NYCC,SoCGainedWithRegen_NYCC);
plot(timeLoopedMin_UDDS,SoCGainedWithRegen_UDDS);
legend('HW-FET','NYCC','UDDS'); xlabel('Time [min]','FontSize',16); ylabel('SoC Gained [%]','FontSize',16);
title('SoC Gained With Regen','FontSize',18);
hold off; % next plot will clear the previous plots if figure(1) is still used

%% check own results with Plett's results

% load resultsGP
% resultsGP_HWFET = resultsGP{4};
% resultsGP_NYCC = resultsGP{1};
% resultsGP_UDDS = resultsGP{2};

% disp('SoC At End BK HW-FET'); disp(driveProfilesResults{1}.battSoCWithRegen(end));
% disp('SoC At End GP HW-FET'); disp(resultsGP_HWFET.batterySOC(end));
% disp('SoC At End BK NYCC'); disp(driveProfilesResults{2}.battSoCWithRegen(end));
% disp('SoC At End GP NYCC'); disp(resultsGP_NYCC.batterySOC(end));
% disp('SoC At End BK UDDS'); disp(driveProfilesResults{3}.battSoCWithRegen(end));
% disp('SoC At End GP UDDS'); disp(resultsGP_UDDS.batterySOC(end));

% figure(7); plot(timeLooped,results.battSoCWithRegen,timeLooped,results.battSoCNoRegen);
% title('State of Charge for the NYCC Speed Profile'); legend('Regen','No Regen');
% xlabel('Time [s]'); ylabel('SoC [%]');

% figure(8); plot(timeLooped,results.battCurrent,timeLooped,resultsGP.current); legend('BK','GP');
% title('Battery Current');
%figure(9); plot(timeLooped,results.actualSpeed, timeLooped,resultsGP.actualSpeed); legend('BK','GP');

% figure(10); plot(timeLooped,results.battVoltagePack); title('Battery Pack Voltage');
% 
% figure(11); plot(timeLooped,results.battVCell); title('Battery Cell Voltage');



% load resultsEKFSoCTrue; % load true SoC for each speed profile

%% plotting EKF vs CC, find RMS error

% fig = figure(2); hold on;
% % plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.battSoCWithRegen,'r','LineWidth',7);
% % plot(driveProfilesResults{1}.timeLooped/60,100*driveProfilesResults{1}.zEKF,'g','LineWidth',2);
% % plot(driveProfilesResults{1}.timeLooped/60,100*ekfTrueSoC_HWFET,'LineWidth',2);
% plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.battSoCWithRegen);
% plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.battSoCWithRegenBias);
% title('SoC Estimation for HW-FET Speed Profile','FontSize',18);
% h_legend = legend('Coulomb Counting','Coulomb Counting With 1% Current Bias','FontSize');
% set(h_legend,'FontSize',14);
% xlabel('Time [min]','FontSize',16); ylabel('SoC [%]','FontSize',16);
% hold off;
% %magnifyOnFigure(fig,'mode','interactive');


fig1 = figure(3); hold on;
plot(driveProfilesResults{2}.timeLooped/60,driveProfilesResults{2}.battSoCWithCurrentSenseError,'LineWidth',3);
plot(driveProfilesResults{2}.timeLooped/60,100*driveProfilesResults{2}.zEKF,'LineWidth',3);
plot(driveProfilesResults{2}.timeLooped/60,driveProfilesResults{2}.battSoCWithRegen,'LineWidth',3,'LineStyle','--');
title('SoC Estimation NYCC Profile','FontSize',18);
h_legend1 = legend('CC With Current Sensor Error','EKF With Current Sensor Error','CC Without Error');
set(h_legend1,'FontSize',16);
xlabel('Time [min]','FontSize',18); ylabel('SoC [%]','FontSize',18);
set(gca,'FontSize',18); % changes font size of axis numbers
hold off;
magnifyOnFigure(fig1,'mode','interactive');

pause % magnifyOnFigure only works for one figure at a time in interactive mode, press any button to continue

% figure(4); plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.sigmaY);
% title('Voltage Error Covariance','FontSize',18);
% xlabel('Time [min]','FontSize',16); ylabel('EKF Voltage Uncertainty','FontSize',16);

fig2 = figure(5); hold on;
plot(driveProfilesResults{2}.timeLooped/60,driveProfilesResults{2}.vEst,'LineWidth',3);
plot(driveProfilesResults{2}.timeLooped/60,driveProfilesResults{2}.battVoltagePack,'LineWidth',3,'LineStyle','--');
title('Estimated vs Simulated Voltage NYCC','FontSize',18); h_legend2 = legend('EKF Estimated Voltage','Simulated Voltage');
set(h_legend2,'FontSize',16);
xlabel('Time [min]','FontSize',18); ylabel('Voltage [V]','FontSize',18);
set(gca,'FontSize',18); % changes font size of axis numbers
magnifyOnFigure(fig2,'mode','interactive');

pause % magnifyOnFigure only works for one figure at a time in interactive mode, press any button to continue

fig3 = figure(6); hold on;
plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.battSoCWithCurrentSenseError,'LineWidth',3);
plot(driveProfilesResults{1}.timeLooped/60,100*driveProfilesResults{1}.zEKF,'LineWidth',3);
plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.battSoCWithRegen,'LineWidth',3,'LineStyle','--');
title('SoC Estimation HWFET Profile','FontSize',18);
h_legend3 = legend('CC With Current Sensor Error','EKF With Current Sensor Error','CC Without Error');
set(h_legend3,'FontSize',16);
xlabel('Time [min]','FontSize',18); ylabel('SoC [%]','FontSize',18);
set(gca,'FontSize',18); % changes font size of axis numbers
hold off;
magnifyOnFigure(fig3,'mode','interactive');

pause % magnifyOnFigure only works for one figure at a time in interactive mode, press any button to continue


% figure(4); plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.sigmaY);
% title('Voltage Error Covariance','FontSize',18);
% xlabel('Time [min]','FontSize',16); ylabel('EKF Voltage Uncertainty','FontSize',16);

fig4 = figure(7); hold on;
plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.vEst,'LineWidth',3);
plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.battVoltagePack,'LineWidth',3,'LineStyle','--');
title('Estimated vs Simulated Voltage HWFET','FontSize',18); h_legend4 = legend('EKF Estimated Voltage','Simulated Voltage');
set(h_legend4,'FontSize',16);
xlabel('Time [min]','FontSize',18); ylabel('Voltage [V]','FontSize',18);
set(gca,'FontSize',18); % changes font size of axis numbers
magnifyOnFigure(fig4,'mode','interactive');



fprintf('RMS Error HWFET SoC EKF: %3.3f \n',findRMSError(driveProfilesResults{1}.battSoCWithRegen/100,driveProfilesResults{1}.zEKF)); % in decimal form
fprintf('RMS Error HWFET Voltage: %3.3f \n',findRMSError(driveProfilesResults{1}.battVoltagePack,driveProfilesResults{1}.vEst));
fprintf('RMS Error NYCC SoC EKF: %3.3f \n',findRMSError(driveProfilesResults{2}.battSoCWithRegen/100,driveProfilesResults{2}.zEKF)); % decimal form
fprintf('RMS Error NYCC Voltage: %3.3f \n',findRMSError(driveProfilesResults{2}.battVoltagePack,driveProfilesResults{2}.vEst));

fprintf('RMS Error HWFET SoC CC: %3.3f \n',findRMSError(driveProfilesResults{1}.battSoCWithRegen/100,driveProfilesResults{1}.battSoCWithCurrentSenseError/100)); % in decimal form
fprintf('RMS Error NYCC SoC CC: %3.3f \n',findRMSError(driveProfilesResults{2}.battSoCWithRegen/100,driveProfilesResults{2}.battSoCWithCurrentSenseError/100)); % in decimal form

%%

% 
% figure(13); hold on;
% %purple = [0.55 0.26 0.51]; % find color using gimp, find RGB values -> divide by 255 to get value between 0 and 1
% % see http://www.tigercolor.com/color-lab/color-theory/color-harmonies.htm
% plot(driveProfilesResults{2}.timeLooped/60,driveProfilesResults{2}.battSoCWithRegen);
% plot(driveProfilesResults{2}.timeLooped/60,100*driveProfilesResults{2}.zEKF);
% plot(driveProfilesResults{2}.timeLooped/60,100*ekfTrueSoC_NYCC);
% title('SoC Estimation for NYCC Speed Profile');
% legend('Coulomb Counting','EKF','True SoC No Noise');
% xlabel('Time [min]'); ylabel('SoC [%]');
% hold off;
% 
% figure(14); hold on;
% plot(driveProfilesResults{3}.timeLooped/60,driveProfilesResults{3}.battSoCWithRegen);
% %orange = [.95 .76 .35];
% plot(driveProfilesResults{3}.timeLooped/60,100*driveProfilesResults{3}.zEKF);
% plot(driveProfilesResults{3}.timeLooped/60,100*ekfTrueSoC_UDDS);
% title('SoC Estimation for UDDS Speed Profile');
% legend('Coulomb Counting','EKF','True SoC No Noise');
% xlabel('Time [min]'); ylabel('SoC [%]');
% hold off;



% figure(15);
% plot(driveProfilesResults{1}.timeLooped,driveProfilesResults{1}.battSoCWithRegen,resultsGP_HWFET.time,resultsGP_HWFET.batterySOC);
% hold on;
% plot(driveProfilesResults{2}.timeLooped,driveProfilesResults{2}.battSoCWithRegen,resultsGP_NYCC.time,resultsGP_NYCC.batterySOC);
% plot(driveProfilesResults{3}.timeLooped,driveProfilesResults{3}.battSoCWithRegen,resultsGP_UDDS.time,resultsGP_UDDS.batterySOC);
% hold off;

% figure(16);
% plot(driveProfilesResults{1}.timeLooped,driveProfilesResults{1}.battCurrent,resultsGP_HWFET.time,resultsGP_HWFET.current);
% 
% figure(17);
% plot(driveProfilesResults{1}.timeLooped,driveProfilesResults{1}.battPower,resultsGP_HWFET.time,resultsGP_HWFET.batteryDemand);
% 
% figure(18);
% plot(driveProfilesResults{1}.timeLooped,driveProfilesResults{1}.battVoltagePack,resultsGP_HWFET.time,resultsGP_HWFET.batteryDemand);

% figure(19);
% plot(timeLoopedMin_HWFET,actSpeedKPH_HWFET);
% hold on;
% plot(timeLoopedMin_HWFET,driveProfilesResults{1}.battCurrent);
% hold off; title('HW-FET Profile'); legend('Actual Speed','Battery Current');
% 
% figure(20);
% plot(timeLoopedMin_UDDS,actSpeedKPH_UDDS);
% hold on;
% plot(timeLoopedMin_UDDS,driveProfilesResults{3}.battCurrent);
% hold off; title('UDDS Profile'); legend('Actual Speed','Battery Current');

% figure(21);
% plot(timeLoopedMin_UDDS,driveProfilesResults{3}.actualAccel);
% hold on;
% plot(timeLoopedMin_NYCC,driveProfilesResults{2}.actualAccel);
% hold off; legend('Actual Acceleration UDDS','Actual Acceleration NYCC');


%% Time To Go And Range Calculations Downhill, Mixed, Uphill Elevation Profiles

% fprintf('Time to Go HWFET: %6.3f min \n',driveProfilesResults{1}.timeToGo/60);
% fprintf('Time to Go NYCC: %6.3f min \n',driveProfilesResults{2}.timeToGo/60);
% fprintf('Time to Go UDDS: %6.3f min \n',driveProfilesResults{3}.timeToGo/60);
% 
% %% Extrapolated Range Calculations
% 
fprintf('Extrapolated Range HWFET: %6.3f km \n',driveProfilesResults{1}.range);
fprintf('Extrapolated Range NYCC: %6.3f km \n',driveProfilesResults{2}.range);
fprintf('Extrapolated Range UDDS: %6.3f km \n',driveProfilesResults{3}.range);
% 
% %% Plot Profile Distance
% 
% figure(22); plot(timeLoopedMin_HWFET,driveProfilesResults{1}.distance); hold on;
% plot(timeLoopedMin_NYCC,driveProfilesResults{2}.distance);
% plot(timeLoopedMin_UDDS,driveProfilesResults{3}.distance); hold off;
% xlabel('Time [min]'); ylabel('Distance [km]'); title('Distance Calculations');
% legend('HWFET','NYCC','UDDS');

%% Elevation, Speed Profile, SoC Plotting

% load('elevationAndDistanceProfileThreeCases'); % loads *ProfileElevAndDistStruct
% figure(10);
% subplot(3,1,1); plot(downhillProfileElevAndDistStruct.distance/1000,downhillProfileElevAndDistStruct.elevation);
% title('Elevation to Speed Profile to SoC','FontSize',18); xlabel('Distance [km]','FontSize',16); ylabel('Elevation [m]','FontSize',16);
% % xlabh = get(gca,'XLabel'); set(xlabh,'Position',get(xlabh,'Position') - [0 0.2 0]); % moves x axis 0.5 to the left
% % ylabh = get(gca,'YLabel'); set(ylabh,'Position',get(ylabh,'Position') - [2 0 0]); % moves y axis 0.5 to the left
% subplot(3,1,2); plot(timeLoopedMin_HWFET,actSpeedKPH_HWFET);
% xlabel('Time [min]','FontSize',16); ylabel('Speed [km/h]','FontSize',16);
% %xlabh = get(gca,'XLabel'); set(xlabh,'Position',get(xlabh,'Position') - [0.05 0 0]); % moves x axis 0.5 to the left
% % ylabh = get(gca,'YLabel'); set(ylabh,'Position',get(ylabh,'Position') - [0.2 0 0]); % moves y axis 0.5 to the left
% subplot(3,1,3); plot(timeLoopedMin_HWFET,ekfTrueSoC_HWFET*100);
% %plot(timeLoopedMin_HWFET,ekfTrueSoC_HWFET*100,timeLoopedMin_HWFET,100*(ekfTrueSoC_HWFET-ekfTrueSoCErrorBounds_HWFET),timeLoopedMin_HWFET,100*(ekfTrueSoC_HWFET+ekfTrueSoCErrorBounds_HWFET));
% xlabel('Time [min]','FontSize',16); ylabel('SoC [%]','FontSize',16); title('EKF','FontSize',16);
% % xlabh = get(gca,'XLabel'); set(xlabh,'Position',get(xlabh,'Position') - [0 0.2 0]); % moves x axis 0.5 to the left
% % ylabh = get(gca,'YLabel'); set(ylabh,'Position',get(ylabh,'Position') - [0.2 0 0]); % moves y axis 0.5 to the left
% % subplot(4,1,4); plot(timeLoopedMin_HWFET,driveProfilesResults{1}.battSoCWithRegen);
% % xlabel('Time [min]'); ylabel('SoC [%]'); title('Coulomb Counting');
% 
% figure(11);
% subplot(3,1,1); plot(mixedProfileElevAndDistStruct.distance/1000,mixedProfileElevAndDistStruct.elevation);
% title('Elevation to Speed Profile to SoC','FontSize',18); xlabel('Distance [km]','FontSize',16); ylabel('Elevation [m]','FontSize',16);
% % xlabh = get(gca,'XLabel'); set(xlabh,'Position',get(xlabh,'Position') - [0 0.2 0]); % moves x axis 0.5 to the left
% % ylabh = get(gca,'YLabel'); set(ylabh,'Position',get(ylabh,'Position') - [2 0 0]); % moves y axis 0.5 to the left
% subplot(3,1,2); plot(timeLoopedMin_UDDS,actSpeedKPH_UDDS);
% xlabel('Time [min]','FontSize',16); ylabel('Speed [km/h]','FontSize',16); 
% % ylabh = get(gca,'YLabel'); set(ylabh,'Position',get(ylabh,'Position') - [0.2 0 0]); % moves y axis 0.5 to the left
% subplot(3,1,3); plot(timeLoopedMin_UDDS,ekfTrueSoC_UDDS*100);
% %plot(timeLoopedMin_NYCC,ekfTrueSoC_NYCC*100,timeLoopedMin_NYCC,100*(ekfTrueSoC_NYCC-ekfTrueSoCErrorBounds_NYCC),timeLoopedMin_NYCC,100*(ekfTrueSoC_NYCC+ekfTrueSoCErrorBounds_NYCC));
% xlabel('Time [min]','FontSize',16); ylabel('SoC [%]','FontSize',16); title('EKF','FontSize',16);
% % xlabh = get(gca,'XLabel'); set(xlabh,'Position',get(xlabh,'Position') - [0 0.2 0]); % moves x axis 0.5 to the left
% % ylabh = get(gca,'YLabel'); set(ylabh,'Position',get(ylabh,'Position') - [0.2 0 0]); % moves y axis 0.5 to the left
% % subplot(4,1,4); plot(timeLoopedMin_NYCC,driveProfilesResults{2}.battSoCWithRegen);
% % xlabel('Time [min]'); ylabel('SoC [%]'); title('Coulomb Counting');
% 
% 
% figure(12);
% subplot(3,1,1); plot(uphillProfileElevAndDistStruct.distance/1000,uphillProfileElevAndDistStruct.elevation);
% title('Elevation to Speed Profile to SoC','FontSize',18); xlabel('Distance [km]','FontSize',16); ylabel('Elevation [m]','FontSize',16);
% % xlabh = get(gca,'XLabel'); set(xlabh,'Position',get(xlabh,'Position') - [0.2 0 0]); % moves x axis 0.5 to the left
% % ylabh = get(gca,'YLabel'); set(ylabh,'Position',get(ylabh,'Position') - [2 0 0]); % moves y axis 0.5 to the left
% subplot(3,1,2); plot(timeLoopedMin_NYCC,actSpeedKPH_NYCC);
% xlabel('Time [min]','FontSize',16); ylabel('Speed [km/h]','FontSize',16); 
% % ylabh = get(gca,'YLabel'); set(ylabh,'Position',get(ylabh,'Position') - [0.2 0 0]); % moves y axis 0.5 to the left
% %subplot(3,1,3); plot(timeLoopedMin_UDDS,ekfTrueSoC_UDDS*100,timeLoopedMin_UDDS,100*(ekfTrueSoC_UDDS-ekfTrueSoCErrorBounds_UDDS),timeLoopedMin_UDDS,100*(ekfTrueSoC_UDDS+ekfTrueSoCErrorBounds_UDDS));
% subplot(3,1,3); plot(timeLoopedMin_NYCC,ekfTrueSoC_NYCC*100);
% xlabel('Time [min]','FontSize',16); ylabel('SoC [%]','FontSize',16); title('EKF','FontSize',16);
% % xlabh = get(gca,'XLabel'); set(xlabh,'Position',get(xlabh,'Position') - [0.2 0 0]); % moves x axis 0.5 to the left
% % ylabh = get(gca,'YLabel'); set(ylabh,'Position',get(ylabh,'Position') - [0.2 0 0]); % moves y axis 0.5 to the left
% % subplot(4,1,4); plot(timeLoopedMin_UDDS,driveProfilesResults{3}.battSoCWithRegen);
% % xlabel('Time [min]'); ylabel('SoC [%]'); title('Coulomb Counting');


%figure(11);
% plot(timeLoopedMin_HWFET,driveProfilesResults{1}.battSoCWithRegen);

% figure(11);
% subplot(1,3,2); plot(mixedProfileElevAndDistStruct.distance,mixedProfileElevAndDistStruct.elevation);
% figure(12);
% subplot(1,3,3); plot(uphillProfileElevAndDistStruct.distance,uphillProfileElevAndDistStruct.elevation);

%plot(driveProfilesResults{2}.timeLooped/60,100*driveProfilesResults{2}.zEKF);
% actSpeedKPH_HWFET
% timeLoopedMin_HWFET


%% plot grade angle info

% figure(20); plot(downhillStruct.HWFET); xlabel('Sample Points','FontSize',16); ylabel('Grade Angle [%]','FontSize',16); title('Grade Angle for HW-FET Downhill Profile','FontSize',18);
% 
% %% check how EKF handles noise, real voltage vs EKF estimated voltage, CC SoC without noise vs EKF est SoC
% speedProfileNum = 3;
% 
% figure(21); plot(driveProfilesResults{speedProfileNum}.timeLooped/60,driveProfilesResults{speedProfileNum}.vEst,driveProfilesResults{speedProfileNum}.timeLooped/60,driveProfilesResults{speedProfileNum}.battVoltagePack);
% legend('EKF Voltage Estimate','Real Battery Voltage');
% 
% figure(22); plot(driveProfilesResults{speedProfileNum}.timeLooped/60,driveProfilesResults{speedProfileNum}.battSoCWithRegen,driveProfilesResults{speedProfileNum}.timeLooped/60,100*driveProfilesResults{speedProfileNum}.zEKF);
% legend('CC SoC No Noise','EKF with Noise SoC');

%% plot eigenvalue of HWFET profile

% figure(50); plot(driveProfilesResults{1}.timeLooped/60,driveProfilesResults{1}.eigValueZk); xlabel('Time [min]','FontSize',16); ylabel('SoC Eigenvalue z-plane','FontSize',16); title('SoC Eigenvalue HWFET Profile','FontSize',18);
% figure(51); plot(driveProfilesResults{2}.timeLooped/60,driveProfilesResults{2}.eigValueZk);
% figure(52); plot(driveProfilesResults{3}.timeLooped/60,driveProfilesResults{3}.eigValueZk); 
