% code updated from Gregory Plett, Battery Management Systems, Volume II, Equivalent-Circuit Methods," Artech House, 
% 2015.

% EKF updated by Bjorn Kvaale for the Revolve battery cell
function [zk,zkBound,yhat,ekfDataStruct] = iterativeEKF(vk,ik,Deltat,ekfDataStruct,OCVAtEvery1PercentSoC)
  
  % Load the cell model parameters
  Q  = ekfDataStruct.Q; % battery capacity at each tested temperature [Ah]
  gamma  = ekfDataStruct.gamma; % hysteresis parameter gamma (unitless)
  M  = ekfDataStruct.M; % hysteresis parameter [V]
  M0 = ekfDataStruct.M0; % instantaneous hysteresis parameter [V]
  tauRC = ekfDataStruct.tauRC; % RC time constant ???, models a simplified Warburg impedance
  R1  = ekfDataStruct.R1; % resistance in the RC parallel circuit [Ohm]
  R0 = ekfDataStruct.R0; % series resistance parameter [Ohm]
  eta = ekfDataStruct.eta; % coulombic efficiency
  
  if ik<0, ik=ik*eta; end; % if charging (ik<0)->eta<1 because of unwanted side reactions, if discharging->eta = 1
  
  % Get stored data from ekfDataStruct
  I = ekfDataStruct.priorI; % I[k-1], previous value of current, initialized to 0
  SigmaX = ekfDataStruct.SigmaX; % initial Kalman filter covariance
  SigmaV = ekfDataStruct.SigmaV; % sensor noise covariance
  SigmaW = ekfDataStruct.SigmaW; % process noise covariance
  xhat = ekfDataStruct.xhat; % state estimate, xhat = [ir hk zk]'
  irIndex = ekfDataStruct.irInd; % intialized as 1 ???
  hkIndex = ekfDataStruct.hkInd; % hysteresis, initialized as 2
  zkIndex = ekfDataStruct.zkInd; % SoC, initialized as 3
  if abs(ik)>Q/100, ekfDataStruct.signIk = sign(ik); end; % see pg 2-13 ESC GP Notes otherwise signIk=0 (signIk initialized to 0)
  signIk = ekfDataStruct.signIk; % signum function, either -1, 0 or 1, initialized to zero
  
  %% EKF: see GP Battery State Estimation Notes pgs 3-42 -> 3-45
  
  % EKF Step 0: Compute Ahat[k-1], Bhat[k-1] ???
  nx = length(xhat); Ahat = zeros(nx,nx); Bhat = zeros(nx,1); % initializate memory ???
  Ahat(zkIndex,zkIndex) = 1; Bhat(zkIndex) = -Deltat/(Q);  % linear approximations using Taylor series
  Ahat(irIndex,irIndex) = diag(tauRC); Bhat(irIndex) = 1-tauRC(:); % linear approximations
  Ahbar  = exp(-abs(I*gamma*Deltat/(Q)));  % hysteresis factor
  Ahat(hkIndex,hkIndex) = Ahbar;
  B = [Bhat, 0*Bhat];
  Bhat(hkIndex) = -abs(gamma*Deltat/(Q))*Ahbar*(1+sign(I)*xhat(hkIndex));
  B(hkIndex,2) = Ahbar-1;
  
  % Step 1a: State estimate/prediction time update, \bar{w}_{k-1} not initialized to zero
  xhat = Ahat*xhat + B*[I; sign(I)]; % just plug the estimates into the state equation
  xhat(hkIndex) = min(1,max(-1,xhat(hkIndex))); % -1<=hk<=1
  xhat(zkIndex) = min(1.05,max(-0.05,xhat(zkIndex))); % find x_hat_k^- \approx f(x_hat_{k-1}^+,u_{k-1},\bar{w}_{k-1}, -0.05<=zk<=1.05

  % Step 1b: Error covariance time update
  %          sigmaminus(k) = Ahat(k-1)*sigmaplus(k-1)*Ahat(k-1)' + ...
  %                          Bhat(k-1)*sigmawtilde*Bhat(k-1)'
  SigmaX = Ahat*SigmaX*Ahat' + Bhat*SigmaW*Bhat'; % see page 3-43, prediction-error covariance equation
  
  % Step 1c: Output estimate, y_hat_k \approx h(x_hat_k^-,uk,vbar_k), see pg 3-44
  % assume best guess is made by using your best estimate so far
  % (x_hat_k^-) and using the mean sensor noise
  hk = xhat(hkIndex);
  iR1k = xhat(irIndex);
  OCVAtZk = findOCVAtZk(OCVAtEvery1PercentSoC,xhat(zkIndex));
  %disp('OCVAtZk'); disp(OCVAtZk);
%   yhat = OCVfromSOCAndTemp(xhat(zkIndex),Tk,model)+M0*signIk+M*hk-R1*iR1k-R0*ik;
    yhat = OCVAtZk+M0*signIk+M*hk-R1*iR1k-R0*ik;
  
  % Step 2a: Estimator gain matrix
  Chatk = zeros(1,nx); % initialize Chat
  Chatk(irIndex) = -R1; % R1 since there is only one parallel RC circuit in the ESC model
  Chatk(hkIndex) = M;
  %Chatk(zkIndex) = dOCVfromSOCtemp(xhat(zkIndex),Tk,model);
  Chatk(zkIndex) = dOCVdSoC(OCVAtEvery1PercentSoC,xhat(zkIndex));
  Dhatk = 1;
  SigmaY = Chatk*SigmaX*Chatk' + Dhatk*SigmaV*Dhatk'; % SigmaY is a constant
  Lk = SigmaX*Chatk'/SigmaY; % hence you can divide by SigmaY instead of doing inverse calculations
  
  % Step 2b: State estimate measurement update
  r = vk - yhat; % vk = yk, innovation/residual  Use to check for sensor errors??
  %if r^2 > 100*SigmaY, Lk(:)=0.0; end  % If there is a measurement error, set Kalman gain=0, go to next iteration, NEES???
  xhat = xhat + Lk*r; % xhat+ = xhat- + Lk*r
  xhat(hkIndex) = min(1,max(-1,xhat(hkIndex))); % Help maintain robustness, ???
  xhat(zkIndex) = min(1.05,max(-0.05,xhat(zkIndex))); % ???
  
  % Step 2c: Error covariance measurement update
  SigmaX = SigmaX - Lk*SigmaY*Lk';
  %   % Q-bump code
  if r^2 > 4*SigmaY % bad voltage estimate by 2 std. deviations, bump Q??? 
    fprintf('Bumping SigmaX\n'); % bump the covariance matrix, simulates additional process noise, helps KF reacquire
    SigmaX(zkIndex,zkIndex) = SigmaX(zkIndex,zkIndex)*ekfDataStruct.Qbump; % Qbump initialized to 5
  end
  [~,S,V] = svd(SigmaX); % singular value decomposition, SigmaX = U*S*V', where U is not important (U = ~), pg 3-39 state est notes GP
  HH = V*S*V'; % H matrix, Joseph form covariance update
  SigmaX = (SigmaX + SigmaX' + HH + HH')/4; % Help maintain robustness, ensures covariance matrices remain symmetric and positive semi-/definite ??
  
  % Save data in ekfDataStruct for the next iteration
  ekfDataStruct.priorI = ik;
  ekfDataStruct.SigmaX = SigmaX;
  ekfDataStruct.xhat = xhat;
  
  % Output of iterativeEKF
  zk = xhat(zkIndex);
  zkBound = 3*sqrt(SigmaX(zkIndex,zkIndex)); % xhat_k^+ +- 3*sqrt(diag(SigmaX)), see slide 3-30 GP State Estimation
end