%% Acknowledgement

% Work presented here based on 4 main papers
% - [1]-[3] Extended Kalman filtering for battery management systems of LiPB-based
%   HEV battery packs by Gregory Plett Parts 1-3
% - [4] Parametrization of a Battery Simulation Model Using Numerical
%   Optimization Methods by Gregory Plett et. al.
%   see http://mocha-java.uccs.edu/BMS2/ for an example of GP's EKF
% - [5] waitbar.m function copyright 2009 Andrew


%% Initialization

clc; clear; close all;

% load dynamic data from cell-testing, contains scripts 1-3, script 1 most
% interesting, scripts 1-3 refer to Equivalent-Circuit Cell Models Lecture, GP's home page
% script 1 runs a UDDS cycle, scripts 2 & 3 ensure that the voltage reaches
% vmin or vmax (using a dither profile at the end)

%load('E2_DYN_15_P05'); T = 15; % data measured at 5 degrees C
load('paramEstimationOutput.mat'); % includes 'timeEKF','iEKF','vEKF','trueSoC','OCVAtEvery1PercentSoC','thetaHatPlus','eta_25','Q_est'
% loads from paramEstimationDynamic.m

% thetaHatPlus(5) = 0.004; % R1, values from ?rjan
% thetaHatPlus(6) = 0.002155249715829; % R0, values from ?rjan
% C1 = 6.1598e4;
% thetaHatPlus(1) = thetaHatPlus(5)*C1; % tau_RC = R1*C1


Deltat = timeEKF(2)-timeEKF(1);
timeEKF = timeEKF-timeEKF(1); % start at t=0
%vEKF = DYNData.script1.voltage';
%trueSoC = DYNData.script1.soc';

% clear figure; figure(1); plot(time,voltage);
% figure(2); plot(time,current);

% Reserve storage for EKF estimate & bounds, used for plotting
socHat = zeros(size(timeEKF)); % SoC prediction
socBound = zeros(size(timeEKF)); % SoC error bound
vEst = zeros(size(timeEKF)); % estimated EKF voltage

%% EKF Initialization
% see ECE5720 Chapter 3 Notes from GP's homepage for EKF implementation

% Nonlinear system dynamics
% xk = f(xk-1,uk-1,wk-1), state equation, wk: process-noise random input
% yk = h(xk,uk,vk), output equation, vk: sensor-noise random input

% Initialize covariance values
% SigmaX0 = diag([1e-6 1e-8 2e-4]); % uncertainty of initial state
SigmaX0 = diag([1e-6 1e-8 1e-6]); % uncertainty of initial state
SigmaW = 2e-1; % Current sensor uncertainty, state equation
SigmaV = 2e-1; % Voltage sensor uncertainty, output equation

% use first voltage & temperature measurements to initialize variables,
% save variables in ekfDataStruct
ekfDataStruct = initializeEKF(SigmaX0,SigmaV,SigmaW,thetaHatPlus,eta_25,Q_est); % SoC0 initialized to 100% at the moment

%% EKF Loop
% enter for loop, update EKF once every sample interval

% TODO: Consider using the waitbar.m function to show percentage of calculations done
%hwait = waitbar(0,'Computing...'); % shows progress bar for EKF calculations, uses waitbar.m function
lengthEKF = length(vEKF);

for k = 1:lengthEKF
    vk = vEKF(k); % "measure" voltage at timestep k
    ik = iEKF(k); % "measure" current at timestep k
    
%     if(k>lengthEKF/2)
%        ik=ik*0.9; 
%     end
    
    % Update SoC and ekfDataStruct,
    [socHat(k),socBound(k),vEst(k),ekfDataStruct] = iterativeEKF(vk,ik,Deltat,ekfDataStruct,OCVAtEvery0Point1PercentSoC);
    % update waitbar periodically, not too often (slow procedure)
%     if mod(k,1000)==0
%        waitbar(k/length(iEKF),hwait); 
%     end;
end
%close(hwait); % close the progress bar

%% Plotting

%close all; % close all previous plots

errorSoC = trueSoC'-socHat;
upperBound = socHat+socBound;
lowerBound = socHat-socBound;
hours = timeEKF/3600; % 3600 seconds in one hour

P = 20; % take every Pth measurement to plot, otherwise takes too long!
socHatPlot = socHat(1:P:lengthEKF);
upperBoundPlot = upperBound(1:P:lengthEKF);
lowerBoundPlot = lowerBound(1:P:lengthEKF);
trueSoCPlot = trueSoC(1:P:lengthEKF);
hoursPlot = hours(1:P:lengthEKF);

figure(1); plot(hoursPlot,trueSoCPlot,hoursPlot,socHatPlot); legend('True SoC','Est SoC');


%close all;
% figHandler = figure(1);
% plot(hours,100*soc,hours,100*socHat,hours,100*(socHat-socBound),hours,100*(socHat+socBound));
% title(['EKF Measured Vs Estimated SoC at T= ' num2str(T) ' ^o C']);
% xlabel('time [h]'); ylabel('SoC [%]');
% legend('Measured SoC','Estimated SoC','SoC Lower Bound','SoC Upper Bound');
% magnifyOnFigure(...
%         figHandler,...
%         'units', 'pixels',...
%         'magnifierShape', 'rectangle',...
%         'initialPositionSecondaryAxes', [326.933 259.189 164.941 100],...
%         'initialPositionMagnifier',     [200 100 15 15],... % [left bottom width height], changes size of rectangle 
%         'mode', 'interactive',...    
%         'displayLinkStyle', 'straight',...        
%         'edgeWidth', 2,...
%         'edgeColor', 'black',...
%         'secondaryAxesFaceColor', [0.91 0.91 0.91]... 
%             ); 

%%

errorUpperBound = errorSoC+socBound;
errorLowerBound = errorSoC-socBound;

figHandler = figure(2);
plot(hours,100*errorSoC,hours,100*errorLowerBound,hours,100*errorUpperBound,'LineWidth',2);
title('Error between Measured and Estimated SoC at T= 25^o C','FontSize',16);
xlabel('Time [h]','FontSize',16); ylabel('Soc Error [%]','FontSize',16);
legend('Error','Error Lower Bound','Error Upper Bound'); 
set(gca,'FontSize',14);
magnifyOnFigure(figHandler,'mode','interactive');

% 
figHandler = figure(7);
plot(hours,100*socHat,hours,100*trueSoC,hours,100*upperBound,hours,100*lowerBound,'LineWidth',2);
legend('Estimated SoC','True SoC','Upper Bound','Lower Bound');
title('Estimated vs True SoC at T=25^o C','FontSize',16); xlabel('Time [hours]','FontSize',16); ylabel('SoC [%]','FontSize',16);
set(gca,'FontSize',14);
%magnifyOnFigure(figHandler,'mode','interactive');

% figHandler = figure(8);
% plot(hours,vEst,hours,vEKF,'LineWidth',2); h_legendV = legend('Estimated Voltage','True Voltage');
% set(h_legendV,'FontSize',14);
% xlabel('Time [hours]','FontSize',16); ylabel('Voltage [V]','FontSize',16); title('Estimated vs True Voltage at T=25^o C','FontSize',16);
% set(gca,'FontSize',16);
% magnifyOnFigure(figHandler,'mode', 'interactive');


errorV = vEKF - vEst;
figure(9);
plot(hours,errorV,'LineWidth',2); xlabel('Time [hours]','FontSize',16);
h_legendError = ylabel('Voltage Error [V]'); set(h_legendError,'FontSize',16);
title('Error Between True And Estimated Voltage at T=25^o C','FontSize',16);
set(gca,'FontSize',14);
magnifyOnFigure(figHandler,'mode','interactive');

%% Testing How Measured SoC Is Found

% SoCMeasured = SoCFromCurrentEtaQ(current,ekfDataStruct,Deltat,T);
% 
% figure(1);
% plot(hours,SoCMeasured,hours,soc);
% legend('SoC Measured BK','Soc Measured GP');

%% Find the error between true and est voltage in [%]

errorVPercent = length(vEKF);

for index=1:length(vEKF)
    errorVPercent = (vEKF(index)-vEst(index))/vEKF(index);
end

